/*
 * parse-graph.h
 *
 *  Created on: Aug 13, 2012
 *      Author: jun
 */

#ifndef ParseGraph_H_
#define ParseGraph_H_

#include "cfg.h"
#include <gecode/int.hh>
#include <limits>

using namespace Gecode;
using namespace std;

const int MAX_INT = numeric_limits<int>::max();

struct Support_Node_Info
{
	int x_pos;
	int y_pos;
	int node_pos;

	Support_Node_Info(int t_x_pos, int t_y_pos, int t_node_pos)
	:x_pos(t_x_pos), y_pos(t_y_pos), node_pos(t_node_pos){};
};

struct Support_Production_Info
{
	Support_Node_Info father;
	Support_Node_Info left_child;
	Support_Node_Info right_child;

	Support_Production_Info(Support_Node_Info t_father, Support_Node_Info t_left_child,
			Support_Node_Info t_right_child)
	:father(t_father), left_child(t_left_child), right_child(t_right_child){};
};

struct Support_Production_Info_List
{
	Support_Production_Info* support_cnf;
	Support_Production_Info_List* next;

	Support_Production_Info_List(Support_Production_Info* t_support_cnf)
	:support_cnf(t_support_cnf), next(NULL){};
};

struct Support_Production_Info_List_Pr
{
	Support_Production_Info_List* support_head_pr;
	Support_Production_Info_List* support_end_pr;

	Support_Production_Info_List_Pr()
	:support_head_pr(NULL), support_end_pr(NULL){};
};

struct Support_For_Node_Info_List
{
	Support_Production_Info* support_cnf;
	Support_For_Node_Info_List* next;
	bool is_left;

	Support_For_Node_Info_List(Support_Production_Info* t_support_cnf, bool t_is_left)
	:support_cnf(t_support_cnf), next(NULL), is_left(t_is_left){};
};

struct Support_For_Node_Info_List_Pr
{
	Support_For_Node_Info_List* support_head_pr;
	Support_For_Node_Info_List* support_end_pr;

	Support_For_Node_Info_List_Pr()
	:support_head_pr(NULL), support_end_pr(NULL){};
};

struct Support_Info_List_Pr_For_Node
{
	Support_For_Node_Info_List_Pr up_support_list;
	Support_For_Node_Info_List_Pr low_support_list;
};

struct Node_In_Parse_Graph
{
	bool is_in_back_up;
	int node_sym;

	int up_support_count;
	int low_support_count;

	Node_In_Parse_Graph(int sym, int t_up_support_count, int t_low_support_count)
	:is_in_back_up(false), node_sym(sym), up_support_count(t_up_support_count),
	 low_support_count(t_low_support_count){};
};

struct Node_Back_Up
{
	Node_In_Parse_Graph* node_pr;
	Node_In_Parse_Graph node;

	Node_Back_Up(Node_In_Parse_Graph* t_node_pr, Node_In_Parse_Graph t_node)
	:node_pr(t_node_pr), node(t_node){};
};

struct Node_With_Pos
{
	int x_pos;
	int node_pos;
	Node_In_Parse_Graph* node_pr;

	Node_With_Pos(int t_x_pos, int t_node_pos, Node_In_Parse_Graph* t_node_pr)
	:x_pos(t_x_pos), node_pos(t_node_pos), node_pr(t_node_pr){};
};

struct My_Range
{
	int r_min;
	int r_max;

	My_Range(int t_r_min, int t_r_max)
	:r_min(t_r_min), r_max(t_r_max){};
};

class ParseGraph
{
public:
	ParseGraph(string cfg_file_name, int t_n,
		const map<string,int>& string_to_nonterminal,
		const map<int,string>& nonterminal_name,
		const int& t_terminal_count);
	~ParseGraph();
	ExecStatus build_parse_graph(Home& home, ViewArray<Int::IntView>& dec_variables, int id);
	ExecStatus filtering_from_update(Space& home, const set<int>& dom_chg_pos,
			ViewArray<Int::IntView>& dec_variables, bool need_back_up = true);
	void push_dummy_node(int prop_id);
	void pop_back_up_nodes(int prop_id);
	void node_remove_up(int x, int y, int node_pos, bool need_back_up);
	void node_remove_low(int x, int y, int node_pos, bool need_back_up);
	void recomputeV(vector<vector<vector<Node_In_Parse_Graph> > >& SV,
			ViewArray<Int::IntView>& dec_variables);
	void compareV(vector<vector<vector<Node_In_Parse_Graph> > > SV);

	const int n;

	CFG* cfg;
	int start_sym;
	int sym_num;
	map<int,string> sym_name;
	map<int,string> terminal_name;

	//  for back track //
	stack<Node_Back_Up> nodes_back_up;
    queue<Node_In_Parse_Graph*> nodes_is_in_back_up;

    map<int,queue<Node_With_Pos> > nodes_lose_up_support;
    map<int,queue<Node_With_Pos> > nodes_lose_low_support;

	//  The table to store the parse graph  V[y][x][node_pos]//
	vector<vector<vector<Node_In_Parse_Graph> > > V;

	vector<map<int,int> > sym_to_node_pos;

	//  store all the support cnf for any node in the graph //
	vector<vector<Support_Info_List_Pr_For_Node> > support_for_node;

	Support_Production_Info_List_Pr all_support;

	bool input_file_error;

	static string debug_file_path;
	static ofstream debug_file;
	static bool need_debug;
	bool first_filter;
	static int debug_count;
	int cst_id;
	static int line_count;
	int prop_id;
};

int ParseGraph::debug_count = 0;
string ParseGraph::debug_file_path = "/home/jun/Work/Rostering/experiments/gecode_debug.txt";
ofstream ParseGraph::debug_file;
bool ParseGraph::need_debug = false;
int ParseGraph::line_count = 0;

ParseGraph::ParseGraph(string cfg_file_name, int t_n,
		const map<string,int>& string_to_terminal,
		const map<int,string>& t_terminal_name,
		const int& t_terminal_count)
:n(t_n)
{
	cfg = new CFG(cfg_file_name, string_to_terminal, t_terminal_name, t_terminal_count);
	input_file_error = cfg->input_file_error;
	if(cfg->input_file_error) return;
	//cfg->print();
	cfg->makeCNF();
	//cfg->printCNF();

	start_sym = cfg->start_sym;
	sym_name = cfg->sym_name;
	sym_num = cfg->sym_count;
	terminal_name = t_terminal_name;

	V.resize(n+1);
	V[0].resize(n);
	for(int i=1; i<=n; i++)
	{
		V[i].resize(n+1-i);
	}

	sym_to_node_pos.resize(n+1);

	support_for_node.resize(n+1);

	if(!debug_file.is_open())
		debug_file.open((char*)debug_file_path.c_str());
	first_filter = true;
}

void ParseGraph::recomputeV(vector<vector<vector<Node_In_Parse_Graph> > >& SV, ViewArray<Int::IntView>& dec_variables)
{
	SV.resize(n+1);
	SV[0].resize(n);
	for(int i=1; i<=n; i++)
	{
		SV[i].resize(n+1-i);
	}

	for(int x=0; x<n; x++)
		SV[0][x] = V[0][0];

	for(int y=1; y<=n; y++)
	{
		int x_up = n-y;
		for(int x=0; x<=x_up; x++)
		{
			SV[y][x] = V[y][0];
		}
	}

	for(int y=0; y<=n; y++)
	{
		int x_up;
		if(y==0) x_up = n;
		else x_up = n+1-y;

		for(int x=0; x<x_up; x++)
		{
			vector<Node_In_Parse_Graph>& node_vec = SV[y][x];
			int vec_size = node_vec.size();
			for(int i=0; i<vec_size; i++)
			{
				node_vec[i].low_support_count = 0;
				node_vec[i].up_support_count = 0;
			}
		}
	}

	SV[n][0][0].up_support_count = 1;

	/// compute the low support count for x=0..n-1 and y=1;
	for(int y=1; y<=1; y++)
	{
		for(int x=0; x<n; x++)
		{
			vector<Node_In_Parse_Graph>& child_node_vec = SV[0][x];
			vector<Node_In_Parse_Graph>& father_node_vec = SV[1][x];

			Int::ViewValues<Int::IntView> var_dom_it(dec_variables[x]);
			while(var_dom_it())
			{
				int cur_val = var_dom_it.val();
				int child_node_pos = sym_to_node_pos[0][cur_val];
				child_node_vec[child_node_pos].low_support_count = 1;

				Support_For_Node_Info_List* cur_sup_list =
						support_for_node[0][child_node_pos].up_support_list.support_head_pr;
				while(cur_sup_list!=NULL)
				{
					int father_node_pos = cur_sup_list->support_cnf->father.node_pos;
					father_node_vec[father_node_pos].low_support_count++;

					cur_sup_list = cur_sup_list->next;
				}

				++var_dom_it;
			}
		}
	}


	/// compute the low support count for x=0..n-1 and y=2..n
	for(int y=2; y<=n; y++)
	{
		int father_y = y;
		vector<Support_Info_List_Pr_For_Node>& support_for_father_node_vec =
				support_for_node[father_y];

		for(int x=0; x<=(n-y); x++)
		{
			vector<Node_In_Parse_Graph>& father_node_vec = SV[father_y][x];
			int vec_size = father_node_vec.size();

			for(int i=0; i<vec_size; i++)
			{
				Support_Info_List_Pr_For_Node& support_for_father_node =
						support_for_father_node_vec[i];

				Support_For_Node_Info_List* low_support_for_father_node =
						support_for_father_node.low_support_list.support_head_pr;

				while(low_support_for_father_node!=NULL)
				{
					Support_Production_Info* support_cnf =
							low_support_for_father_node->support_cnf;
					Support_Node_Info& left_child_node = support_cnf->left_child;
					Support_Node_Info& right_child_node = support_cnf->right_child;
					int left_child_node_pos = left_child_node.node_pos;
					int left_child_x = x;
					int left_child_y = left_child_node.y_pos;
					int right_child_node_pos = right_child_node.node_pos;
					int right_child_x = x+left_child_y;
					int right_child_y = y-left_child_y;

					if(right_child_x<=(n-right_child_y))
					{
						if(SV[left_child_y][left_child_x][left_child_node_pos].low_support_count>0)
						{
							if(SV[right_child_y][right_child_x][right_child_node_pos].low_support_count>0)
							{
								father_node_vec[i].low_support_count++;
							}
						}

					}

					low_support_for_father_node = low_support_for_father_node->next;
				}
			}
		}
	}

	//  compute the up support count for x=0..n-1 and y=2..n //
	for(int y=n; y>1; y--)
	{
		int father_y = y;
		vector<Support_Info_List_Pr_For_Node>& support_for_father_node_vec =
				support_for_node[father_y];

		for(int x=0; x<=(n-y); x++)
		{
			vector<Node_In_Parse_Graph>& father_node_vec = SV[father_y][x];
			int vec_size = father_node_vec.size();

			for(int i=0; i<vec_size; i++)
			{
		    	if(father_node_vec[i].up_support_count==0)
		    	{
		    		father_node_vec[i].low_support_count = 0;
		    	}
				else
				{
					Support_Info_List_Pr_For_Node& support_for_father_node =
							support_for_father_node_vec[i];

					Support_For_Node_Info_List* low_support_for_father_node =
							support_for_father_node.low_support_list.support_head_pr;

					while(low_support_for_father_node!=NULL)
					{
						Support_Production_Info* support_cnf =
								low_support_for_father_node->support_cnf;
						Support_Node_Info& left_child_node = support_cnf->left_child;
						Support_Node_Info& right_child_node = support_cnf->right_child;
						int left_child_node_pos = left_child_node.node_pos;
						int left_child_x = x;
						int left_child_y = left_child_node.y_pos;
						int right_child_node_pos = right_child_node.node_pos;
						int right_child_x = x+left_child_y;
						int right_child_y = y-left_child_y;

						if(right_child_x<=(n-right_child_y))
						{
							if(SV[left_child_y][left_child_x][left_child_node_pos].low_support_count>0)
							{
								if(SV[right_child_y][right_child_x][right_child_node_pos].low_support_count>0)
								{
									SV[left_child_y][left_child_x][left_child_node_pos].up_support_count++;
									SV[right_child_y][right_child_x][right_child_node_pos].up_support_count++;
								}
							}

						}

						low_support_for_father_node = low_support_for_father_node->next;
					}
				}
			}
		}
	}

	//  compute the up support count for x=0..n-1 and y=1 //
	for(int y=1; y>0; y--)
	{
		int father_y = y;
		vector<Support_Info_List_Pr_For_Node>& support_for_father_node_vec =
				support_for_node[father_y];

		for(int x=0; x<=(n-y); x++)
		{
			vector<Node_In_Parse_Graph>& father_node_vec = SV[father_y][x];
			int vec_size = father_node_vec.size();

			for(int i=0; i<vec_size; i++)
			{
				if(father_node_vec[i].up_support_count==0)
				{
					father_node_vec[i].low_support_count = 0;
				}
				else
				{
					Support_Info_List_Pr_For_Node& support_for_father_node =
							support_for_father_node_vec[i];

					Support_For_Node_Info_List* low_support_for_father_node =
							support_for_father_node.low_support_list.support_head_pr;

					while(low_support_for_father_node!=NULL)
					{
						Support_Production_Info* support_cnf =
								low_support_for_father_node->support_cnf;
						int left_child_node_pos = support_cnf->left_child.node_pos;

						if(SV[0][x][left_child_node_pos].low_support_count>0)
						{
							SV[0][x][left_child_node_pos].up_support_count++;
						}

						low_support_for_father_node = low_support_for_father_node->next;
					}
				}
			}
		}
	}

	for(int x=0; x<n; x++)
	{
		int v_size =SV[0][x].size();
		for(int i=0; i<v_size; i++)
		{
			if(SV[0][x][i].up_support_count==0)
				SV[0][x][i].low_support_count = 0;
		}
	}
}

void ParseGraph::compareV(vector<vector<vector<Node_In_Parse_Graph> > > SV)
{
	bool find_error = false;

	for(int y=0; y<=n; y++)
	{
		int x_up;
		if(y==0) x_up = n-1;
		else x_up = n-y;

		for(int x=0; x<=x_up; x++)
		{
			int v_size = V[y][x].size();
			for(int i=0; i<v_size; i++)
			{
				if(V[y][x][i].low_support_count!=SV[y][x][i].low_support_count)
				{
					cout << V[y][x][i].low_support_count << endl;
					cout << SV[y][x][i].low_support_count << endl;

					find_error = true;
				}

				if(V[y][x][i].up_support_count!=SV[y][x][i].up_support_count)
				{
					cout << V[y][x][i].up_support_count << endl;
					cout << SV[y][x][i].up_support_count << endl;

					find_error = true;
				}
			}
		}
	}

	if(!find_error)  cout << "compare is ok" << endl;
	else cout << "The propagation is wrong here" << endl;
}

ExecStatus ParseGraph::build_parse_graph(Home& home, ViewArray<Int::IntView>& dec_variables, int id)
{
	cst_id = id;

	// compute the union of all domains
	Int::ViewRanges<Int::IntView> var_rng_it[n];
	for(int x=0; x<n; x++)
	{
		var_rng_it[x].init(dec_variables[x]);
	}

	queue<My_Range> exist_rng;

	while(true)
	{
		bool new_rng_exist = false;
		int rec_min = MAX_INT;
		int rec_max = -1;

		for(int x=0; x<n; x++)
		{
			if(var_rng_it[x]())
			{
				new_rng_exist = true;

				int cur_min = var_rng_it[x].min();
				int cur_max = var_rng_it[x].max();
				if(rec_min>cur_min)  rec_min = cur_min;
				if(rec_max<cur_max)  rec_max = cur_max;

				++var_rng_it[x];
			}
		}

		if(!new_rng_exist)  break;
		else
		{
			if(!exist_rng.empty())
			{
				My_Range& pre_rng = exist_rng.back();
				if(rec_min<=pre_rng.r_max && rec_max>pre_rng.r_max)
					pre_rng.r_max = rec_max;
				else if(rec_min<=rec_max)
					exist_rng.push(My_Range(rec_min,rec_max));
			}
			else if(rec_min<=rec_max)
			{
				exist_rng.push(My_Range(rec_min,rec_max));
			}
		}
	}

	//  compute the whole support information for x=0 and y=0 //
	for(int y=0; y<1; y++)
	{
		int child_node_count = -1;
		int child_node_pos;
		int father_node_count = -1;
		int father_node_pos;
		map<int,int>& father_to_node_pos = sym_to_node_pos[1];
		map<int,int>::iterator int_to_int_it;

		bool find_support = false;

		while(!exist_rng.empty())
		{
			My_Range& cur_rng = exist_rng.front();
			int cur_min = cur_rng.r_min;
			int cur_max = cur_rng.r_max;

			for(int cur_val=cur_min; cur_val<=cur_max; cur_val++)
			{
				bool cur_val_find_support = false;

				vector<int>& father_sym_vec = cfg->terminal_to_head_sym_vec[cur_val];
				int vec_size = father_sym_vec.size();

				if(vec_size>0)
				{
					cur_val_find_support = true;

					child_node_count++;
					child_node_pos = child_node_count;
	                sym_to_node_pos[0][cur_val] = child_node_pos;
	                V[0][0].push_back(Node_In_Parse_Graph(cur_val,vec_size,1));
	                support_for_node[0].push_back(Support_Info_List_Pr_For_Node());

					for(int i=0; i<vec_size; i++)
					{
						int father_sym = father_sym_vec[i];
						int_to_int_it = father_to_node_pos.find(father_sym);
						if(int_to_int_it==father_to_node_pos.end())
						{
							father_node_count++;
							sym_to_node_pos[1][father_sym] = father_node_count;
							father_node_pos = father_node_count;
							V[1][0].push_back(Node_In_Parse_Graph(father_sym,0,0));
							support_for_node[1].push_back(Support_Info_List_Pr_For_Node());
						}
						else  father_node_pos = int_to_int_it->second;

						V[1][0][father_node_pos].low_support_count++;

						Support_Production_Info* new_support =
								new Support_Production_Info(
								Support_Node_Info(0,1,father_node_pos),
								Support_Node_Info(0,0,child_node_pos),
			                    Support_Node_Info(-1,-1,-1));

/*						cout << "father " << V[1][0][father_node_pos].node_sym << ": x=0, y=1, node_pose=" << father_node_pos << "; "
								<< "child " << V[0][0][child_node_pos].node_sym << ": x=0, y=0, node_pose=" << child_node_pos << endl;
*/
						if(all_support.support_end_pr==NULL)
						{
							all_support.support_end_pr =
									new Support_Production_Info_List(new_support);
							all_support.support_head_pr = all_support.support_end_pr;
						}
						else
						{
							all_support.support_end_pr->next =
									new Support_Production_Info_List(new_support);
							all_support.support_end_pr = all_support.support_end_pr->next;
						}

						Support_For_Node_Info_List_Pr& low_support_list_for_father =
								support_for_node[1][father_node_pos].low_support_list;

						if(low_support_list_for_father.support_end_pr==NULL)
						{
							low_support_list_for_father.support_end_pr =
									new Support_For_Node_Info_List(new_support,true);
							low_support_list_for_father.support_head_pr =
									low_support_list_for_father.support_end_pr;
						}
						else
						{
							low_support_list_for_father.support_end_pr->next =
									new Support_For_Node_Info_List(new_support,true);
							low_support_list_for_father.support_end_pr =
									low_support_list_for_father.support_end_pr->next;
						}

						Support_For_Node_Info_List_Pr& up_support_list_for_child =
								support_for_node[0][child_node_pos].up_support_list;

						if(up_support_list_for_child.support_end_pr==NULL)
						{
							up_support_list_for_child.support_end_pr =
									new Support_For_Node_Info_List(new_support,true);
							up_support_list_for_child.support_head_pr =
									up_support_list_for_child.support_end_pr;
						}
						else
						{
							up_support_list_for_child.support_end_pr->next =
									new Support_For_Node_Info_List(new_support,true);
							up_support_list_for_child.support_end_pr =
									up_support_list_for_child.support_end_pr->next;
						}
					}
				}

				if(cur_val_find_support)  find_support = true;
				else
				{
					for(int x=0; x<n; x++)
					{
/*						cout << "Var_" << x << " removes value "
								<< cur_val << endl;
*/
						GECODE_ME_CHECK(dec_variables[x].nq(home,cur_val));
					}
				}
			}

			exist_rng.pop();
		}

		if(!find_support)  return ES_FAILED;
	}

	//  compute the low support count for x=0 and y=1..n //
	for(int y=2; y<=n; y++)
	{
		int father_y = y;
		vector<Node_In_Parse_Graph>& father_node_vec = V[father_y][0];
		map<int,int>& father_to_node_pos = sym_to_node_pos[father_y];
		int father_node_count = -1;
		int father_node_pos;
		int father_sym;
		vector<Support_Info_List_Pr_For_Node>& support_for_father_node =
				support_for_node[father_y];

		map<int,int>::iterator int_to_int_it;

		for(int k=1; k<y; k++)
		{
			int left_child_y = k;
			vector<Node_In_Parse_Graph>& left_child_node_vec = V[left_child_y][0];
			int left_vec_size = left_child_node_vec.size();
			vector<Support_Info_List_Pr_For_Node>& support_for_left_child_node =
					support_for_node[left_child_y];

			int right_child_y = y-k;
			map<int,int>& right_child_to_node_pos = sym_to_node_pos[right_child_y];
			vector<Support_Info_List_Pr_For_Node>& support_for_right_child_node =
					support_for_node[right_child_y];

			for(int left_child_node_pos=0; left_child_node_pos<left_vec_size;
					left_child_node_pos++)
			{
				Node_In_Parse_Graph& left_child_node = left_child_node_vec[left_child_node_pos];
				int left_child_sym = left_child_node.node_sym;

				Support_For_Node_Info_List_Pr& up_support_list_for_left_child =
						support_for_left_child_node[left_child_node_pos].up_support_list;

				CNFProdListPr& left_child_sym_to_cnf_prod =
						cfg->left_child_sym_to_cnf_prod[left_child_sym];
				CNFProdList* cur_left_cnf_prod_list = left_child_sym_to_cnf_prod.head_pr;
				while(cur_left_cnf_prod_list!=NULL)
				{
					CNFProduction* cur_cnf_prod = cur_left_cnf_prod_list->CNFProd_pr;
					father_sym = cur_cnf_prod->gen_sym;
					if((y<n)||(father_sym==start_sym))
					{
						int right_child_sym = cur_cnf_prod->from_second;

						int_to_int_it = right_child_to_node_pos.find(right_child_sym);
						if(int_to_int_it!=right_child_to_node_pos.end())
						{
							int right_child_node_pos = int_to_int_it->second;

							int_to_int_it = father_to_node_pos.find(father_sym);
							if(int_to_int_it!=father_to_node_pos.end())
							{
								father_node_pos = int_to_int_it->second;
							}
							else
							{
								father_node_count++;
								father_node_pos = father_node_count;
								father_to_node_pos[father_sym] = father_node_pos;

								father_node_vec.push_back(Node_In_Parse_Graph(father_sym,0,0));
								support_for_father_node.push_back(Support_Info_List_Pr_For_Node());
							}

							Node_In_Parse_Graph& father_node = father_node_vec[father_node_pos];
							father_node.low_support_count++;

							Support_Production_Info* new_support =
									new Support_Production_Info(
									Support_Node_Info(0,father_y,father_node_pos),
									Support_Node_Info(0,left_child_y,left_child_node_pos),
				                    Support_Node_Info(left_child_y,right_child_y,right_child_node_pos));

/*							cout << "father " << V[father_y][0][father_node_pos].node_sym << ": x=0, y=" << father_y << ", node_pose=" << father_node_pos << "; "
									<< "left " << V[left_child_y][0][left_child_node_pos].node_sym << ": x=0, y=" << left_child_y << ", node_pose=" << left_child_node_pos << "; "
									<< "right " << V[right_child_y][0][right_child_node_pos].node_sym << ": x=" << left_child_y << ", y=" << right_child_y << ", node_pose=" << right_child_node_pos << endl;
*/

							all_support.support_end_pr->next =
									new Support_Production_Info_List(new_support);
							all_support.support_end_pr = all_support.support_end_pr->next;

							/*vector<Support_Info_List_Pr_For_Node>& support_for_father_node =
									support_for_node[father_y];*/

							Support_For_Node_Info_List_Pr& low_support_list_for_father =
									support_for_father_node[father_node_pos].low_support_list;

							if(low_support_list_for_father.support_end_pr==NULL)
							{
								low_support_list_for_father.support_end_pr =
										new Support_For_Node_Info_List(new_support,true);
								low_support_list_for_father.support_head_pr =
										low_support_list_for_father.support_end_pr;
							}
							else
							{
								low_support_list_for_father.support_end_pr->next =
										new Support_For_Node_Info_List(new_support,true);
								low_support_list_for_father.support_end_pr =
										low_support_list_for_father.support_end_pr->next;
							}

							if(up_support_list_for_left_child.support_end_pr==NULL)
							{
								up_support_list_for_left_child.support_end_pr =
										new Support_For_Node_Info_List(new_support,true);
								up_support_list_for_left_child.support_head_pr =
										up_support_list_for_left_child.support_end_pr;
							}
							else
							{
								up_support_list_for_left_child.support_end_pr->next =
										new Support_For_Node_Info_List(new_support,true);
								up_support_list_for_left_child.support_end_pr =
										up_support_list_for_left_child.support_end_pr->next;
							}

							Support_For_Node_Info_List_Pr& up_support_list_for_right_child =
									support_for_right_child_node[right_child_node_pos].up_support_list;

							if(up_support_list_for_right_child.support_end_pr==NULL)
							{
								up_support_list_for_right_child.support_end_pr =
										new Support_For_Node_Info_List(new_support,false);
								up_support_list_for_right_child.support_head_pr =
										up_support_list_for_right_child.support_end_pr;
							}
							else
							{
								up_support_list_for_right_child.support_end_pr->next =
										new Support_For_Node_Info_List(new_support,false);
								up_support_list_for_right_child.support_end_pr =
										up_support_list_for_right_child.support_end_pr->next;
							}
						}
					}

					cur_left_cnf_prod_list = cur_left_cnf_prod_list->next;
				}
			}
		}
	}

	if(V[n][0].size()==0)  return ES_FAILED;

	for(int x=1; x<n; x++)
	{
		V[0][x] = V[0][0];
	}

	for(int y=1; y<n; y++)
	{
		int x_up = n-y;
		for(int x=1; x<=x_up; x++)
		{
			V[y][x] = V[y][0];
		}
	}

	V[n][0][0].up_support_count = 1;

	//  compute the up support count for x=0..n-1 and y=2..n //
	for(int y=n; y>1; y--)
	{
		int x_up = n-y;

		int father_y = y;
		vector<Node_In_Parse_Graph>& father_node_vec = V[father_y][0];
		vector<Support_Info_List_Pr_For_Node>& support_for_father_node_vec =
				support_for_node[father_y];
		int vec_size = father_node_vec.size();

		for(int i=0; i<vec_size; i++)
		{
			Support_Info_List_Pr_For_Node& support_for_father_node =
					support_for_father_node_vec[i];

			Support_For_Node_Info_List* low_support_for_father_node =
					support_for_father_node.low_support_list.support_head_pr;

			while(low_support_for_father_node!=NULL)
			{
				Support_Production_Info* support_cnf =
						low_support_for_father_node->support_cnf;
				Support_Node_Info& left_child_node = support_cnf->left_child;
				Support_Node_Info& right_child_node = support_cnf->right_child;
				int left_child_node_pos = left_child_node.node_pos;
				int left_child_y = left_child_node.y_pos;
				int right_child_node_pos = right_child_node.node_pos;
				int right_child_x = left_child_y;
				int right_child_y = y-left_child_y;

				for(int x_shift=0; x_shift<=x_up; x_shift++)
				{
					Node_In_Parse_Graph& shift_father_node =
							V[y][x_shift][i];
					if(shift_father_node.up_support_count>0)
					{
						V[left_child_y][x_shift][left_child_node_pos].up_support_count++;
						V[right_child_y][x_shift+right_child_x][right_child_node_pos].up_support_count++;
					}
					else  shift_father_node.low_support_count = 0;
				}

				low_support_for_father_node = low_support_for_father_node->next;
			}
		}
	}

	//  compute the up support count for x=0..n-1 and y=1 //
	for(int y=1; y>0; y--)
	{
		int x_up = n-y;

		int father_y = y;
		vector<Node_In_Parse_Graph>& father_node_vec = V[father_y][0];
		vector<Support_Info_List_Pr_For_Node>& support_for_father_node_vec =
				support_for_node[father_y];
		int vec_size = father_node_vec.size();

		for(int i=0; i<vec_size; i++)
		{
			Support_Info_List_Pr_For_Node& support_for_father_node =
					support_for_father_node_vec[i];

			Support_For_Node_Info_List* low_support_for_father_node =
					support_for_father_node.low_support_list.support_head_pr;

			for(int x_shift=0; x_shift<=x_up; x_shift++)
			{
				Node_In_Parse_Graph& shift_father_node =
						V[y][x_shift][i];
				if(shift_father_node.up_support_count==0)
				{
					shift_father_node.low_support_count = 0;

					Support_For_Node_Info_List* temp_list = low_support_for_father_node;

					while(temp_list!=NULL)
					{
						Support_Production_Info* support_cnf =
								temp_list->support_cnf;
						Support_Node_Info& left_child_node_info = support_cnf->left_child;
						int left_child_node_pos = left_child_node_info.node_pos;
						int left_child_y = left_child_node_info.y_pos;

						Node_In_Parse_Graph& left_child_node =
								V[left_child_y][x_shift][left_child_node_pos];
						left_child_node.up_support_count--;

						if(left_child_node.up_support_count==0)
						{
/*							cout << "Var_" << x_shift << " removes value "
									<< left_child_node.node_sym << endl;

*/
							left_child_node.low_support_count=0;

							GECODE_ME_CHECK(dec_variables[x_shift].
									nq(home,left_child_node.node_sym));
						}

						temp_list = temp_list->next;
					}
				}
			}
		}
	}

	// filtering caused by non-existed values
	set<int> dom_chg_pos;
	for(int i=0; i<n; i++)
	{
		dom_chg_pos.insert(i);
	}

	if(filtering_from_update(home, dom_chg_pos, dec_variables, false)==ES_FAILED)
		return ES_FAILED;

	first_filter = false;

	if(need_debug)
	{
		line_count++;
		debug_file << "line " << line_count << ": " << cst_id << " finish building parse graph!" << endl;


		line_count++;
		debug_file << "line " << line_count << ": " << cst_id << " The current domain is as following: " << endl;


		for(int chg_pos=0; chg_pos<n; chg_pos++)
		{
			line_count++;
			debug_file << "line " << line_count << ": " << cst_id << " Domain of var_" << chg_pos << ": ";

			Int::ViewValues<Int::IntView> var_dom_it(dec_variables[chg_pos]);
			while(var_dom_it())
			{
				int cur_val = var_dom_it.val();
				debug_file << cur_val << " ";
				++var_dom_it;
			}

			debug_file << endl;
		}
	}

/*	cout << "low support is " << V[8][0][0].low_support_count << endl;
	cout << "up support is " << V[8][0][0].up_support_count << endl;

	cout << "finish building parse graph!" << endl << endl;

	cout << "The current domain is as following: " << endl;
	for(int chg_pos=0; chg_pos<n; chg_pos++)
	{
		cout << "Domain of var_" << chg_pos << ": ";

		Int::ViewValues<Int::IntView> var_dom_it(dec_variables[chg_pos]);
		while(var_dom_it())
		{
			int cur_val = var_dom_it.val();
			cout << cur_val << " ";
			++var_dom_it;
		}

		cout << endl;
	}*/

	return ES_OK;
}

ExecStatus ParseGraph::filtering_from_update(Space& home, const set<int>& dom_chg_pos,
		ViewArray<Int::IntView>& dec_variables, bool need_back_up)
{
/*	debug_count++;

	cout << "constraint id " << cst_id << " filtering counting: " << debug_count << endl;

	vector<vector<vector<Node_In_Parse_Graph> > > SV;
	recomputeV(SV,dec_variables);*/

	if(!nodes_lose_up_support.empty())
		nodes_lose_up_support.clear();

	if(!nodes_lose_low_support.empty())
		nodes_lose_low_support.clear();

	set<int>::iterator set_int_it;
	for(set_int_it=dom_chg_pos.begin(); set_int_it!=dom_chg_pos.end(); set_int_it++)
	{
		int chg_pos = *set_int_it;

		vector<Node_In_Parse_Graph>& cur_sym_node_vec = V[0][chg_pos];
		int vec_size = cur_sym_node_vec.size();
		Node_In_Parse_Graph* cur_node_pr;

		Int::ViewValues<Int::IntView> var_dom_it(dec_variables[chg_pos]);
		int cur_val = var_dom_it.val();

		for(int cur_node_pos=0; cur_node_pos<vec_size; cur_node_pos++)
		{
			cur_node_pr = &cur_sym_node_vec[cur_node_pos];

			// dispose pruned values //
			if(cur_val!=cur_node_pr->node_sym)
			{
				if(cur_node_pr->up_support_count>0)
				{
					if(need_back_up)
					{
						if(!cur_node_pr->is_in_back_up)
						{
							nodes_back_up.push(Node_Back_Up(cur_node_pr,*cur_node_pr));
							nodes_is_in_back_up.push(cur_node_pr);

							cur_node_pr->is_in_back_up = true;
						}
					}

					cur_node_pr->up_support_count = 0;
					cur_node_pr->low_support_count = 0;

/*					if(need_debug)
					{
						line_count++;
						debug_file << "line " << line_count << ": " << cst_id << " Var_"
								<< chg_pos << " has removed value " << cur_node_pr->node_sym
							       << " in previous propagation!" << endl;
					}*/

/*					cout << "Var_" << chg_pos << " has removed value " << cur_node_pr->node_sym
							<< " in previous propagation!" << endl;
*/
					node_remove_up(chg_pos, 0, cur_node_pos, need_back_up);
				}
			}
			else
			{
				++var_dom_it;

				if(var_dom_it())  cur_val = var_dom_it.val();
				else cur_val = -1;
			}
		}
	}

	//  update low support upwards //
	map<int,queue<Node_With_Pos> >::iterator int_to_node_queue_it;
	while(!nodes_lose_low_support.empty())
	{
		int_to_node_queue_it = nodes_lose_low_support.begin();
		int cur_y = int_to_node_queue_it->first;
		queue<Node_With_Pos>& temp_nodes_lose_low_support = int_to_node_queue_it->second;

		while(!temp_nodes_lose_low_support.empty())
		{
			Node_With_Pos& temp_node_with_pos = temp_nodes_lose_low_support.front();
			Node_In_Parse_Graph* cur_node_pr = temp_node_with_pos.node_pr;
			if(cur_node_pr->up_support_count>0)
			{
				if(cur_y==n)  return ES_FAILED;

				int cur_x = temp_node_with_pos.x_pos;
				int cur_node_pos = temp_node_with_pos.node_pos;

				cur_node_pr->up_support_count = 0;
				node_remove_up(cur_x, cur_y, cur_node_pos, need_back_up);
			}

			temp_nodes_lose_low_support.pop();
		}

		nodes_lose_low_support.erase(int_to_node_queue_it);
	}

	//  update up support downwards //
	while(!nodes_lose_up_support.empty())
	{
		int_to_node_queue_it = nodes_lose_up_support.end();
		int_to_node_queue_it--;
		int cur_y = int_to_node_queue_it->first;
		queue<Node_With_Pos>& temp_nodes_lose_up_support = int_to_node_queue_it->second;

		if(cur_y==0)
		{
			while(!temp_nodes_lose_up_support.empty())
			{
				Node_With_Pos& temp_node_with_pos = temp_nodes_lose_up_support.front();
				Node_In_Parse_Graph* cur_node_pr = temp_node_with_pos.node_pr;
				if(cur_node_pr->low_support_count>0)
				{
					cur_node_pr->low_support_count = 0;

					int cur_x = temp_node_with_pos.x_pos;
					int cur_node_pos = temp_node_with_pos.node_pos;
					int cur_sym = V[0][cur_x][cur_node_pos].node_sym;

/*					cout << "Var_" << cur_x << " removes value " << cur_sym << endl;

					int t_sym = 5;
					int t_x = 3;
					int t_y = 5;
					int t_node_pos = sym_to_node_pos[t_y][t_sym];

					cout << "low support is " << V[t_y][t_x][t_node_pos].low_support_count << endl;
					cout << "up support is " << V[t_y][t_x][t_node_pos].up_support_count << endl;
*/
					GECODE_ME_CHECK(dec_variables[cur_x].nq(home,cur_sym));
				}

				temp_nodes_lose_up_support.pop();
			}
		}
		else
		{
			while(!temp_nodes_lose_up_support.empty())
			{
				Node_With_Pos& temp_node_with_pos = temp_nodes_lose_up_support.front();
				Node_In_Parse_Graph* cur_node_pr = temp_node_with_pos.node_pr;
				if(cur_node_pr->low_support_count>0)
				{
					int cur_x = temp_node_with_pos.x_pos;
					int cur_node_pos = temp_node_with_pos.node_pos;

					cur_node_pr->low_support_count = 0;

					node_remove_low(cur_x, cur_y, cur_node_pos, need_back_up);
				}

				temp_nodes_lose_up_support.pop();
			}
		}

		nodes_lose_up_support.erase(int_to_node_queue_it);
	}

	////////////////////////////////////////////

//    compareV(SV);

//	cout << "finish one update!" << endl << endl;

	if(need_debug&&(!first_filter))
	{
		line_count++;
		debug_file << "line " << line_count << ": " << cst_id << " finish one update!" << endl;

		line_count++;
		debug_file << "line " << line_count << ": " << cst_id << " The current domain is as following: " << endl;

		for(int chg_pos=0; chg_pos<n; chg_pos++)
		{
			line_count++;
			debug_file << "line " << line_count << ": " << cst_id << " Domain of var_"
					<< chg_pos << ": ";

			Int::ViewValues<Int::IntView> var_dom_it(dec_variables[chg_pos]);
			while(var_dom_it())
			{
				int cur_val = var_dom_it.val();
				debug_file << cur_val << " ";
				++var_dom_it;
			}

			debug_file << endl;
		}
	}

	return ES_FIX;
}

void ParseGraph::node_remove_low(int x, int y, int node_pos, bool need_back_up)
{
/*	cout << "Remove downwards: node " << V[y][x][node_pos].node_sym
			<< " at x=" << x << " y=" << y << " node_pos=" << node_pos << endl;
*/
	vector<Support_Info_List_Pr_For_Node>& support_for_father_node =
			support_for_node[y];

	Support_For_Node_Info_List* low_support_list_for_father_node =
			support_for_father_node[node_pos].low_support_list.support_head_pr;

	if(y==1)
	{
		while(low_support_list_for_father_node!=NULL)
		{
			Support_Production_Info* cur_low_support =
					low_support_list_for_father_node->support_cnf;

			int child_node_pos = cur_low_support->left_child.node_pos;
			Node_In_Parse_Graph& child_node = V[0][x][child_node_pos];

			if(child_node.up_support_count>0)
			{
				if(need_back_up)
				{
					if(!child_node.is_in_back_up)
					{
						nodes_back_up.push(
								Node_Back_Up(&child_node,child_node));
						nodes_is_in_back_up.push(&child_node);

						child_node.is_in_back_up = true;
					}
				}

				child_node.up_support_count--;
				if(child_node.up_support_count==0)
				{
					nodes_lose_up_support[0].push(
							Node_With_Pos(x,child_node_pos,&child_node));
				}
			}

			low_support_list_for_father_node =
					low_support_list_for_father_node->next;
		}
	}
	else
	{
		while(low_support_list_for_father_node!=NULL)
		{
			Support_Production_Info* cur_low_support =
					low_support_list_for_father_node->support_cnf;

			Support_Node_Info& left_child_node_info = cur_low_support->left_child;
			int left_child_node_pos = left_child_node_info.node_pos;
			int left_child_x = x;
			int left_child_y = left_child_node_info.y_pos;
			Support_Node_Info& right_child_node_info = cur_low_support->right_child;
			int right_child_node_pos = right_child_node_info.node_pos;
			int right_child_x = x+left_child_y;
			int right_child_y = right_child_node_info.y_pos;

			Node_In_Parse_Graph& left_child_node =
					V[left_child_y][left_child_x][left_child_node_pos];

			if(left_child_node.up_support_count>0)
			{
				Node_In_Parse_Graph& right_child_node =
						V[right_child_y][right_child_x][right_child_node_pos];

				if(right_child_node.up_support_count>0)
				{
					if(need_back_up)
					{
						if(!left_child_node.is_in_back_up)
						{
							nodes_back_up.push(
									Node_Back_Up(&left_child_node,left_child_node));
							nodes_is_in_back_up.push(&left_child_node);

							left_child_node.is_in_back_up = true;
						}

						if(!right_child_node.is_in_back_up)
						{
							nodes_back_up.push(
									Node_Back_Up(&right_child_node,right_child_node));
							nodes_is_in_back_up.push(&right_child_node);

							right_child_node.is_in_back_up = true;
						}
					}

					left_child_node.up_support_count--;
					if(left_child_node.up_support_count==0)
					{
						nodes_lose_up_support[left_child_y].push(
								Node_With_Pos(left_child_x,left_child_node_pos,&left_child_node));
					}

					right_child_node.up_support_count--;
					if(right_child_node.up_support_count==0)
					{
						nodes_lose_up_support[right_child_y].push(
								Node_With_Pos(right_child_x,right_child_node_pos,&right_child_node));
					}
				}
			}

			low_support_list_for_father_node =
					low_support_list_for_father_node->next;
		}
	}
}

void ParseGraph::node_remove_up(int x, int y, int node_pos, bool need_back_up)
{
/*	cout << "Remove upwards: node " << V[y][x][node_pos].node_sym
			<< " at x=" << x << " y=" << y << " node_pos=" << node_pos << endl;
*/
	vector<Support_Info_List_Pr_For_Node>& support_for_child_node =
			support_for_node[y];

	Support_For_Node_Info_List* up_support_list_for_child_node =
			support_for_child_node[node_pos].up_support_list.support_head_pr;

	if(y==0)
	{
		while(up_support_list_for_child_node!=NULL)
		{
			Support_Production_Info* cur_up_support =
					up_support_list_for_child_node->support_cnf;

			int father_node_pos = cur_up_support->father.node_pos;
			Node_In_Parse_Graph& father_node = V[1][x][father_node_pos];
			if(father_node.low_support_count>0)
			{
				if(need_back_up)
				{
					if(!father_node.is_in_back_up)
					{
						nodes_back_up.push(
								Node_Back_Up(&father_node,father_node));
						nodes_is_in_back_up.push(&father_node);

						father_node.is_in_back_up = true;
					}
				}

				father_node.low_support_count--;
				if(father_node.low_support_count==0)
				{
					nodes_lose_low_support[1].push(
							Node_With_Pos(x,father_node_pos,&father_node));
				}
			}

			up_support_list_for_child_node =
					up_support_list_for_child_node->next;
		}
	}
	else
	{
		while(up_support_list_for_child_node!=NULL)
		{
			Support_Production_Info* cur_up_support =
					up_support_list_for_child_node->support_cnf;

			Support_Node_Info& father_node_info = cur_up_support->father;
			int father_node_pos = father_node_info.node_pos;
			int father_y = father_node_info.y_pos;
			int father_x;
			int child_node_pos;
			int child_y = father_y-y;
			int child_x;

			if(up_support_list_for_child_node->is_left)
			{
				father_x = x;
				child_x = x+y;
				Support_Node_Info& right_child_node_info = cur_up_support->right_child;
				child_node_pos = right_child_node_info.node_pos;
			}
			else
			{
				father_x = x-child_y;
				child_x = father_x;
				Support_Node_Info& left_child_node_info = cur_up_support->left_child;
				child_node_pos = left_child_node_info.node_pos;
			}

			if((father_x>=0)&&((father_y+father_x)<=n))
			{
				Node_In_Parse_Graph& father_node = V[father_y][father_x][father_node_pos];

				if(father_node.low_support_count>0)
				{
					Node_In_Parse_Graph& child_node =
							V[child_y][child_x][child_node_pos];

					if(child_node.up_support_count>0)
					{
						if(need_back_up)
						{
							if(!father_node.is_in_back_up)
							{
								nodes_back_up.push(
										Node_Back_Up(&father_node,father_node));
								nodes_is_in_back_up.push(&father_node);

								father_node.is_in_back_up = true;
							}

							if(!child_node.is_in_back_up)
							{
								nodes_back_up.push(
										Node_Back_Up(&child_node,child_node));
								nodes_is_in_back_up.push(&child_node);

								child_node.is_in_back_up = true;
							}
						}

						father_node.low_support_count--;
						if(father_node.low_support_count==0)
						{
							nodes_lose_low_support[father_y].push(
									Node_With_Pos(father_x,father_node_pos,&father_node));
						}

						child_node.up_support_count--;
						if(child_node.up_support_count==0)
						{
							nodes_lose_up_support[child_y].push(
									Node_With_Pos(child_x,child_node_pos,&child_node));
						}
					}
				}
			}

			up_support_list_for_child_node =
					up_support_list_for_child_node->next;
		}
	}
}

void ParseGraph::push_dummy_node(int prop_id)
{
	nodes_back_up.push(Node_Back_Up(NULL, Node_In_Parse_Graph(-1, -1, prop_id)));

	while(!nodes_is_in_back_up.empty())
	{
		Node_In_Parse_Graph* cur_node_pr = nodes_is_in_back_up.front();
		cur_node_pr->is_in_back_up = false;

		nodes_is_in_back_up.pop();
	}
}

void ParseGraph::pop_back_up_nodes(int prop_id)
{
	while(!nodes_is_in_back_up.empty())
	    nodes_is_in_back_up.pop();

	while(!nodes_back_up.empty())
	{
		Node_Back_Up& cur_node_back = nodes_back_up.top();
		Node_In_Parse_Graph& cur_node = cur_node_back.node;

		if(cur_node.node_sym>=0)
		{
			*(cur_node_back.node_pr) = cur_node;
		}
		else if(cur_node.low_support_count==prop_id)
		{
			nodes_back_up.pop();
			break;
		}
		else if(cur_node.low_support_count<prop_id)
		{
//			cout << "actually back track to propagator id " << cur_node.low_support_count << endl;
			break;
		}

		nodes_back_up.pop();
	}
}

ParseGraph::~ParseGraph()
{
	map<int,int>::iterator int_to_int_it;
	for(int y=0; y<=n; y++)
	{
		int vec_size = V[y][0].size();
		for(int i=0; i<vec_size; i++)
		{
			Support_Info_List_Pr_For_Node& cur_support = support_for_node[y][i];

			Support_For_Node_Info_List* cur_list =
					cur_support.low_support_list.support_head_pr;
			while(cur_list!=NULL)
			{
				Support_For_Node_Info_List* temp_list = cur_list->next;
				delete cur_list;
				cur_list = temp_list;
			}

			cur_list = cur_support.up_support_list.support_head_pr;
			while(cur_list!=NULL)
			{
				Support_For_Node_Info_List* temp_list = cur_list->next;
				delete cur_list;
				cur_list = temp_list;
			}
		}
	}

	Support_Production_Info_List* cur_list = all_support.support_head_pr;
	while(cur_list!=NULL)
	{
		delete cur_list->support_cnf;

		Support_Production_Info_List* temp_list = cur_list->next;
		delete cur_list;
		cur_list = temp_list;
	}

	delete cfg;
}

#endif /* ParseGraph_H_ */
